function CRB = CRB_Sparse_Arrays_Liu_Vaidyanathan( S, theta_bar, p_s_over_p_n, SNAPSHOTS )
%CRB_Sparse_Arrays_Liu_Vaidyanathan: The CRB expression proposed in Theorem 2 of [a]
% [a]	C.-L. Liu and P. P. Vaidyanathan, 
%		Cramer-Rao Bounds for Coprime and Other Sparse Arrays, which Find More Sources than Sensors,
%		Digital Signal Processing.
%       http://dx.doi.org/10.1016/j.dsp.2016.04.011  
%
% Written by Chun-Lin Liu
% E-mail : cl.liu@caltech.edu
% Website: http://systems.caltech.edu/dsp/students/clliu/CRB.html
% 
% Last revised on July 6, 2016
%
%   Inputs:
%       1) S (column vector): the array configuration as an integer set
%       2) theta_bar (column vector): true normalized DOAs -0.5 <= theta_bar < 0.5
%       3) p_s_over_p_n (column vector): source power/noise power, positive reals
%       4) SNAPSHOTS (scalar): the number of snapshots
%
%   Outputs:
%       1) CRB (matrix): the associated CRB, which is a DxD positive semidefinite
%       matrix.
%
%	Example 1 and 2 of [a]:
%		S_nested = [1; 2; 3; 4; 5; 6; 12; 18; 24; 30];
%		S_super_nested = [1; 3; 5; 8; 10; 12; 18; 24; 29; 30];
%		theta_bar = [0.1; 0.2];
%		p_s_over_p_n = [1; 1];
%		SNAPSHOTS = 500;
%		CRB_nested = CRB_Sparse_Arrays_Liu_Vaidyanathan( S_nested, theta_bar, p_s_over_p_n, SNAPSHOTS );
%		CRB_super_nested = CRB_Sparse_Arrays_Liu_Vaidyanathan( S_super_nested, theta_bar, p_s_over_p_n, SNAPSHOTS );

% Array manifold matrix on S
V_S = exp(2i * pi * S * theta_bar.');

P = diag(p_s_over_p_n);

R_S = V_S * P * V_S' + eye(length(S));

[SS, SS_] = ndgrid(S);
Diff = SS(:) - SS_(:);
D = unique(Diff);

J = zeros(length(Diff), length(D));
for ii = 1 : length(D)
    J(:, ii) = (Diff == D(ii));
end

M = ( (J' / kron(R_S.', R_S) ) * J)^0.5;

V_D = exp(2i * pi * D * theta_bar.');

% Evaluate the rank of the ACM matrix
Ac = [diag(D)*V_D, V_D, D == 0];
rank_ACM = rank(Ac);
if ( rank_ACM ~= size(Ac, 2) )
	warning('Ac does not have full column rank. The CRB expression is divergent.')
end

G_0 = M * diag(D) * V_D * P;

M_W_D = M * [V_D, (D == 0)];

Pi_M_W_D_perp = eye(length(D)) - (M_W_D / (M_W_D'*M_W_D)) * M_W_D';

CRB = inv(G_0' * Pi_M_W_D_perp * G_0) / (4*pi^2*SNAPSHOTS);

end

