function CRB = CRB_Sparse_Arrays_Liu_Vaidyanathan_large_SNR( S, theta_bar, p_s_over_p_n, SNAPSHOTS )
%CRB_Sparse_Arrays_Liu_Vaidyanathan_large_SNR: The asymptotic CRB expressions for large SNR
% See Theorem 3 and 4 of [a]
% [a]	C.-L. Liu and P. P. Vaidyanathan, 
%		Cramer-Rao Bounds for Coprime and Other Sparse Arrays, which Find More Sources than Sensors,
%		Digital Signal Processing.
%       http://dx.doi.org/10.1016/j.dsp.2016.04.011  
%
% Written by Chun-Lin Liu
% E-mail : cl.liu@caltech.edu
% Website: http://systems.caltech.edu/dsp/students/clliu/CRB.html
% 
% Last revised on July 6, 2016
%
%   Inputs:
%       1) S (column vector): the array configuration as an integer set
%       2) theta_bar (column vector): true normalized DOAs -0.5 <= theta_bar < 0.5
%       3) p_s_over_p_n (column vector): source power/noise power, positive reals
%       4) SNAPSHOTS (scalar): the number of snapshots
%
%   Outputs:
%       1) CRB (matrix): the associated CRB, which is a DxD positive semidefinite
%       matrix.
%
%	Sample codes for Fig. 3 of [a]:
%		S = [1; 2; 3; 6];
%		D = 4;
%		theta_bar = -0.49 + 0.9*(0:D-1).'/D;
%		SNRdB = 0;
%		p_s_over_p_n = 10^(SNRdB/10); % equal-power sources
%		SNAPSHOTS = 500;
%		CRB = CRB_Sparse_Arrays_Liu_Vaidyanathan_large_SNR( S, theta_bar, p_s_over_p_n, SNAPSHOTS );



% Array manifold matrix
V_S = exp(2i * pi * S * theta_bar.');

[SS, SS_] = ndgrid(S);
Diff = SS(:) - SS_(:);
D = unique(Diff);

J = zeros(length(Diff), length(D));
for ii = 1 : length(D)
    J(:, ii) = (Diff == D(ii));
end

LEN_S = length(S);


V_D = exp(2i * pi * D * theta_bar.');
W_D = [V_D, (D == 0)];

V_S_V_S_H = V_S * V_S';

rank_V_S = rank( V_S );

if ( rank_V_S == LEN_S )
    % V_S has full row rank
    M_inf = ( (J' / kron(conj(V_S_V_S_H), V_S_V_S_H)) * J)^0.5;
    
    G_inf = M_inf * diag(D) * V_D;
    
    M_W_D = M_inf * [V_D, (D == 0)];
    
    Pi_M_W_D_perp = eye(length(D)) - (M_W_D / (M_W_D'*M_W_D)) * M_W_D';
    
    S_mat = G_inf' * Pi_M_W_D_perp * G_inf;
    
    CRB = inv( S_mat ) / (4*pi^2*SNAPSHOTS);
else
    % V_S does not have full row rank
    [EV, EW] = eig(V_S_V_S_H);
    
    [ewsort, II] = sort( abs(diag(EW)), 'descend' );
    
    EV = EV(:, II);
    
    
    Us = EV(:, 1:rank_V_S);
    Un = EV(:, rank_V_S+1:end);
    
    U_n_U_n_H = Un * Un';
    U_s_Lambda_inv_U_s_H = Us * diag( 1./ewsort(1:rank_V_S) ) * Us';
    
    M_inf = (J' * ( kron( U_s_Lambda_inv_U_s_H.', U_n_U_n_H  ) + ...
                    kron( U_n_U_n_H.', U_s_Lambda_inv_U_s_H ) ) * J)^0.5;
    
    G_inf = M_inf * diag(D) * V_D;
    
    M_inf_W_D = M_inf * W_D;
    
    Pi_M_inf_W_D_perp = eye(length(D)) - (M_inf_W_D / (M_inf_W_D'*M_inf_W_D)) * M_inf_W_D';
    
    UU = M_inf_W_D / (M_inf_W_D'*M_inf_W_D);
    u = UU(:, end);
    
    S_mat = G_inf' * Pi_M_inf_W_D_perp * G_inf + G_inf' * (u * u') * G_inf / (norm(u)^2);
    
    CRB = inv( S_mat ) / (4*pi^2*SNAPSHOTS*p_s_over_p_n);
    
end






end

